
{===============================================================}
{								}
{	Net Vampire manager					}
{	a code by Lake Unanimated / lake@lakeofsoft.com		}
{	with help of Alex Shovkoplyas and Alexey Surkis		}
{								}
{								}
{	Copyright (c) 2000 Lake of Soft, Ltd			}
{	All Rights Reserved					}
{								}
{===============================================================}
{$B-}

unit NVMan;

interface

uses
  Classes, IniFiles, ExtCtrls;

type
  TIniType  = (ftJobs, ftPartial, ftSites, ftState);

  TDataType = (dtString, dtInteger, dtReal);

  TJobPhase = (jphStop, jphWait, jphQueue, jphDial, jphConnect, jphTransfer, jphSuccess, jphFail, jphNoFile, jphCyclic);

  TJobProperty = (
    // from NVJ
    jpLocation, jpTitle, jpLocalFile, jpLocalFolder, jpStartTime, jpPriority, jpRegular,
    jpPeriod, jpPerionUnits, jpOnlyNewer, jpMultiCopies, jpConnPerFile, jpRetryIfSize, jpMinFileSize,
    jpAllowConcurrPerFile, jpStrip, jpReferer, jpUserAgent, jpIgnoreDateChange, jpRestoreDirs,
    jpTag, // <--- Ident
    // from NVC
    jpState, jpFileSize, jpFileTime, jpAttempt, jpFinalName, jpIsHTML,
    // from NVP
    jpPartialSize);

  TJobProperties = set of TJobProperty;

  TIniNameType = record
    rName : string;
    rType : tDataType;
  end;

const
  cFolderNames: array [tIniType] of string =
    ('Jobs', 'Partial', 'Sites', 'State');

  cJobExtensions: array [tIniType] of string =
    ('nvj', 'nvp', 'nvs', 'nvc');

  cIniNamesTypes: array[TJobProperty] of tIniNameType =
   ( // NVJ
     (rName: 'LocID';           rType: dtString),
     (rName: 'Title';           rType: dtString),
     (rName: 'LocalFile';       rType: dtString),
     (rName: 'LocalFolder';     rType: dtString),
     (rName: 'StartTime';       rType: dtReal),
     (rName: 'Priority';        rType: dtInteger),
     (rName: 'Regular';         rType: dtInteger),
     (rName: 'Period';          rType: dtInteger),
     (rName: 'PeriodUnits';     rType: dtInteger),
     (rName: 'OnlyNewer';       rType: dtInteger),
     (rName: 'MultiCopies';     rType: dtInteger),
     (rName: 'ConnPerFile';     rType: dtInteger),
     (rName: 'RetryIfSize';     rType: dtInteger),
     (rName: 'MinFileSize';     rType: dtInteger),
     (rName: 'AllowConcurrPerFile';     rType: dtInteger),
     (rName: 'Strip';           rType: dtInteger),
     (rName: 'Referer';         rType: dtString),
     (rName: 'UserAgent';       rType: dtString),
     (rName: 'IgnoreDateChange';rType: dtInteger),
     (rName: 'RestoreDirs';     rType: dtInteger),
     (rName: 'Tag';             rType: dtString),
     // NVC
     (rName: 'State';           rType: dtInteger),
     (rName: 'FileSize';        rType: dtInteger),
     (rName: 'FileTime';        rType: dtReal),
     (rName: 'Attempt';         rType: dtInteger),
     (rName: 'FinalName';       rType: dtString),
     (rName: 'IsHTML';          rType: dtInteger),
     // NVP
     (rName: 'Partial size';    rType: dtInteger)
   );

type
  TNVManager = class;

  TNVJob = class(tObject)
  private
    FSaved   : Boolean;
    FMan     : TNVManager;
    FIndex   : string;
    FIniFiles: array[TIniType] of TIniFile;
    FCachedProps: array[TJobProperty] of Variant;
    procedure _SetIniValue(AIndex: TJobProperty; const Value: Variant);
    function  _GetIniValue(AIndex: TJobProperty): Variant;
    procedure SetIniValue(AIniType: TIniType; ADataType: TDataType; const ASectionName, AValueName: string; const Value: Variant);
    function  GetIniValue(AIniType: TIniType; ADataType: TDataType; const ASectionName, AValueName: string): Variant;
  protected
    constructor CreateFromFile(AMan: TNVManager; const AIndex: string);
  public
    constructor Create(AMan: TNVManager; const AIndex: string);
    destructor Destroy; override;
    procedure Save;
    //
    property Saved: Boolean read FSaved;
    property IniValue[AIniType: TIniType; ADataType: TDataType; const ASectionName, AValueName: string]: Variant read GetIniValue write SetIniValue;
    property MyIndex: string read fIndex;
    property JobProperty[AIndex: TJobProperty]: Variant read _GetIniValue write _SetIniValue; default;
  end;

  TJobEvent = procedure(Sender: TObject; const AIndex: String) of object;
  TJobChangePropsEvent = procedure(Sender: TObject; Job: TNVJob; const Props: TJobProperties) of object;

  TNVManager = class(tComponent)
  private
    fRootFolder         : string;
    fPartialFolder      : String;
    fJobs               : tList;
    fOnReloadJobs       : tNotifyEvent;
    FOnJobChange        : TJobChangePropsEvent;
    FOnDeleteJob        : TJobEvent;
    FOnNewJob           : TJobEvent;
    FRefreshInterval    : Integer;
    fRefreshTimer       : tTimer;
    fVampire            : Variant;
    fIsRunning          : Boolean;
    FTag                : String;
    FCanReload          : Boolean;
    function GetTheFolder(AIndex: tIniType): string;
    procedure ClearJobs;
    function AddJobFromFile(const FileName: string): String;
    function GetCount: Integer;
    function GetNVJob(const AIndex: string): TNVJob;
    function GetNVJobByIndex(AIndex: Integer): TNVJob;
    procedure SetRefreshInterval(const Value: Integer);
    procedure SetAutoRefresh(const Value: Boolean);
    function GetAutoRefresh: Boolean;
    function GetNetVampirePath: string;
    procedure CheckForNewAndDeletedJobs;
    function GetVampireRunning: Boolean;

    procedure MyOnTimer(Sender: tObject);

    procedure DoReloadJobs;
    procedure DoDeleteJob(const AIndex: String);
    procedure DoNewJob(const AIndex: String);
  public
    constructor Create(aOwner: tComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    //
    function ReloadJobsInfo: Integer;
    function AddJob(const aURL: string; InitialState: TJobPhase; const ATag: String = ''): string;
    procedure DeleteJob(const AIndex: string; DeleteFiles: Boolean = False);
    function JobIsHere(const AIndex: string): string;
    procedure RunVampire;
    //
    property Count: Integer read GetCount;
    property Jobs[const AIndex: string]: TNVJob read GeTNVJob; default;
    property JobByIndex[AIndex: Integer]: TNVJob read GeTNVJobByIndex;
    property TheFolder[AIndex: tIniType]: string read GetTheFolder;
    property RootFolder: string read fRootFolder write fRootFolder;
    property VampireIsRunning: Boolean read GetVampireRunning;
  published
    property RefreshInterval: Integer read fRefreshInterval write SetRefreshInterval;
    property AutoRefresh    : Boolean read GetAutoRefresh write SetAutoRefresh;
    property DefaultTag     : string read FTag write FTag;
    // events
    property OnReloadJobs: tNotifyEvent read fOnReloadJobs write fOnReloadJobs;
    property OnJobChange : TJobChangePropsEvent read FOnJobChange write FOnJobChange;
    property OnDeleteJob : TJobEvent read FOnDeleteJob write FOnDeleteJob;
    property OnNewJob    : TJobEvent read FOnNewJob write FOnNewJob;
  end;

procedure Register;

implementation

uses
  Windows, SysUtils, ComObj, Registry;

function RegReadString(const KeyName, ValueName, DefValue: string;
		       AKey: HKEY = HKEY_LOCAL_MACHINE): string;
begin
  with tRegistry.Create do
  try
    RootKey := AKey;
    if OpenKey(KeyName, False) then Result := ReadString(ValueName)
                               else Result := DefValue;
    CloseKey();
  finally
    Free;
  end;
end;

function RegReadInteger(const KeyName, ValueName: string; DefValue: Integer;
                        AKey: HKEY = HKEY_LOCAL_MACHINE): Integer;
begin
  with tRegistry.Create do
  try
    RootKey := AKey;
    if OpenKey(KeyName, False) then Result := ReadInteger(ValueName)
                               else Result := DefValue;
    CloseKey();
  finally
    Free;
  end;
end;

function RegWriteInteger(const KeyName, ValueName: string; Value: Integer;
                         AKey: HKEY = HKEY_LOCAL_MACHINE): string;
begin
  with tRegistry.Create do try
    RootKey := AKey;
    if OpenKey(KeyName, False) then WriteInteger(ValueName, Value);
    CloseKey();
  finally
    Free();
  end;
end;

const
  strGenIDFirstDigit    = '4';
  strDataPath           = 'Data';
  strVampireDefPath     = 'C:\Program Files\NetVampire';
  strVampireEXE         = 'Vampire.exe';
  strVampireNewJob      = 'Vampire.NewJob';

  strNV_SHARED_MEM      = 'NV_SHARED_MEM';

  sRegKey_GenID         = 'Software\Net Vampire\System';
  sRegName_GenID        = 'NextID_NVMan';

  sRegKey_Partial       = sRegKey_GenID;
  sRegName_Partial      = 'PartialDir';

  sRegKey_VampirePath   = 'SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\Vampire.exe';
  sRegKey_VampirePath2  = 'SOFTWARE\Net Vampire';
  sRegKey_VampirePath3  = 'SOFTWARE\Classes\CLSID\{DC3F3002-E83E-11D1-8146-9911CA6CE53D}\LocalServer32';
  sRegName_VampirePath  = '';
  sRegName_VampirePath2 = 'InstallDir1';
  sRegName_VampirePath3  = '';

  sIni_Settings         = 'SETTINGS';
  sIni_Locations        = 'LOCATIONS';
  sIni_State            = 'STATE';

  iInitialRefresh       = 2000; // 2 sec
  iMinRefresh           = 500;  // 0,5 sec

  DefTag                = 'NVMan';
  EmptyTag              = 'NV';
  Seps: array[Boolean] of String = ( '._', '.' );

{ TNVManager }

function TNVManager.AddJob(const aURL: string; InitialState: TJobPhase; const ATag: String{ = ''}): string;

  function GenID(ID: Integer): string;
  begin
    Result := strGenIDFirstDigit + IntToHex(ID, 7);
  end;

var
  I: Integer;
  J: TNVJob;
begin
  Randomize;
  I := Random(MaxWord);
  repeat
    Inc(i);
  until not FileExists(TheFolder[ftJobs] + GenID(i) + '.' + cJobExtensions[ftJobs]);
  J := TNVJob.Create(Self, GenID(i));
  fJobs.Add(J);
  with J do
  begin
    JobProperty[jpState] := Ord(InitialState);
    fIniFiles[ftJobs].WriteString(sIni_Settings, cIniNamesTypes[jpLocation].rName, GenID(i + 1));
    JobProperty[jpLocation] := aURL;
    if ATag='' then
      JobProperty[jpTag] := FTag else
      JobProperty[jpTag] := ATag;
    Result := MyIndex;
  end;
//  DoReloadJobs;
end;

function TNVManager.AddJobFromFile(const FileName: string): String;
var
  J: TNVJob;
begin
  Result := Copy(ExtractFileName(FileName), 1, 8);
  // fixed by Lake - 2000/05/29
  if (Result <> '') and (Result[Length(Result)] = '.') then Result := Copy(Result, 1, 7); 
  J := TNVJob.CreateFromFile(Self, Result);
  FJobs.Add(J);
  DoNewJob(J.MyIndex);
  DoReloadJobs;
end;

procedure TNVManager.ClearJobs;
var
  i: Integer;
begin
  for i := 0 to fJobs.Count - 1 do TNVJob(fJobs[i]).Free;
  fJobs.Clear;
end;

constructor TNVManager.Create;
begin
  inherited;
  fRefreshInterval := iInitialRefresh;  // 2 sec
  fRefreshTimer    := TTimer.Create(nil);
  fRefreshTimer.Enabled := False;
  fRefreshTimer.OnTimer := MyOnTimer;

  fJobs       := tList.Create;
  FTag        := DefTag;
  FCanReload  := True;
end;

procedure TNVManager.DeleteJob(const AIndex: string; DeleteFiles: Boolean{ = False});
var
  i: tIniType;
  Index: Integer;
  Job: TNVJob;
  Saved: Boolean;
  JobIndex: String;
begin
  Index:=fJobs.IndexOf(Jobs[AIndex]);
  Job:=Jobs[AIndex];
  Saved:=Job.FSaved;
  JobIndex:=Job.MyIndex;
  Job.Free;
  FJobs.Delete(Index);

  if DeleteFiles then
    for i := Low(tIniType) to High(tIniType) do
      DeleteFile(TheFolder[i] + AIndex + Seps[Saved] + cJobExtensions[i]);

  DoDeleteJob(JobIndex);
  DoReloadJobs;
end;

destructor TNVManager.Destroy;
begin
  fRefreshTimer.OnTimer:=nil;
  fRefreshTimer.Free;
  ClearJobs;
  fJobs.Free;
  inherited;
end;

function TNVManager.GetAutoRefresh: Boolean;
begin
  Result := fRefreshTimer.Enabled;
end;

function TNVManager.GetCount: Integer;
begin
  Result := FJobs.Count;
end;

{Added by Alex Shovkoplyas for compatibility with Net Vampire 3.2}
function TNVManager.GetNetVampirePath: string;
var
  p: pChar;
begin
  Result := RegReadString(sRegKey_VampirePath, sRegName_VampirePath, strVampireDefPath);
  if (Result = '') or not FileExists(Result) then
  begin
    Result := RegReadString(sRegKey_VampirePath2, sRegName_VampirePath2, '', HKEY_CURRENT_USER) + '\' + strVampireEXE;
    if (Result = '') or not FileExists(Result) then begin
      // added 26 jul 2001
      // Lake
      Result := RegReadString(sRegKey_VampirePath3, sRegName_VampirePath3, '');
      p := pchar(Result);
      Result := AnsiExtractQuotedStr(p, '"');
      //
      if (Result = '') or not FileExists(Result) then
	raise Exception.Create('This program requires Net Vampire 3.0 or higher.'+#10+'Download it for free from WWW.NETVAMPIRE.COM');
    end;	
  end;
end;

function TNVManager.GetNVJob(const AIndex: string): TNVJob;
var
  i: Integer;
  S: string;
begin
  Result := nil;
  S := UpperCase(AIndex);
  for i := 0 to Count - 1 do
    if (TNVJob(fJobs[i]).fIndex = S) then
    begin
      Result := TNVJob(fJobs[i]);
      Break;
    end;
end;

function TNVManager.GeTNVJobByIndex(AIndex: Integer): TNVJob;
begin
  Result := TNVJob(FJobs[AIndex]);
end;

function TNVManager.GetTheFolder(AIndex: tIniType): string;
begin
  if AIndex<>ftPartial then
    Result := RootFolder + cFolderNames[AIndex] + '\' else
    Result := fPartialFolder;  // Added for compatibility with Net Vampire 3.3
end;

function TNVManager.JobIsHere(const AIndex: string): string;
begin
  Result := '';
  with Jobs[AIndex] do
    if (JobProperty[jpState] = jphSuccess) then Result := JobProperty[jpFinalName];
end;

procedure TNVManager.Loaded;
begin
  if csDesigning in ComponentState then Exit;
   
  fRootFolder := ExtractFileDir(GetNetVampirePath) + '\' + strDataPath + '\';
  fPartialFolder:=RegReadString(sRegKey_Partial, sRegName_Partial, '', HKEY_CURRENT_USER);
  if fPartialFolder='' then
    fPartialFolder:=RootFolder + cFolderNames[ftPartial] + '\';

  fIsRunning  := VampireIsRunning;
  ReloadJobsInfo;
end;

procedure TNVManager.MyOnTimer(Sender: tObject);
var
  i: Integer;
  j: TJobProperty;
  V1, V2: Variant;
  Props: TJobProperties;
  Job: TNVJob;
begin
  CheckForNewAndDeletedJobs;
  if Assigned(FOnJobChange) then
  begin
    i:=0;
    while i<FJobs.Count do
    begin
      Props:=[];
      Job:=JobByIndex[i];
      with Job do
      begin
        for j := Low(TJobProperty) to High(TJobProperty) do
        begin
	  V1 := fCachedProps[j];
          V2 := JobProperty[j]; // force cache
	  if (not VarIsEmpty(V1)) and (V1 <> V2) then
            Include(Props, j);
        end;
      end;
      if Job.Saved and (Props<>[]) then FOnJobChange(Self, Job, Props);
      if FJobs.IndexOf(Job)>=0 then Inc(i);
    end;
  end;
  fIsRunning := VampireIsRunning;
end;

procedure TNVManager.CheckForNewAndDeletedJobs;
var
  F: tSearchRec;
  I: Integer;
  Job: TNVJob;
  JobIndex: String;
  ReloadNeeded: Boolean;
begin
  ReloadNeeded := False;
  FCanReload   := False;
  try
    // check for deleted jobs
    i:=0;
    while i<FJobs.Count do
    begin
      Job := JobByIndex[i];
      with Job do
      begin
        if not FileExists(TheFolder[ftJobs]+FIndex+Seps[FSaved]+cJobExtensions[ftJobs]) then
        begin
          ReloadNeeded := True;
          //DoDeleteJob(Job);
          DeleteJob(FIndex, False);
        end;
      end;
      if FJobs.IndexOf(Job)>=0 then Inc(i);
    end;

    // check for new job
    I := FindFirst(TheFolder[ftJobs] + '*.' + cJobExtensions[ftJobs], faArchive + faReadOnly + faHidden + faSysFile, F);
    try
      while (I = 0) do
      begin
	JobIndex:=UpperCase(Copy(F.Name, 1, 8));
	// fixed by Lake - 2000/05/29
	if (JobIndex <> '') and (JobIndex[Length(JobIndex)] = '.') then JobIndex := Copy(JobIndex, 1, 7);
        if Jobs[JobIndex]=nil then
        begin
          ReloadNeeded := True;
          Jobs[AddJobFromFile(TheFolder[ftJobs]+F.Name)];
          //DoNewJob(Job);
        end;
        I := FindNext(F);
      end;
    finally
      FindClose(F);
    end;
  finally
    FCanReload:=True;
    if ReloadNeeded then DoReloadJobs;
  end;
end;

function TNVManager.ReloadJobsInfo: Integer;
var
  F: tSearchRec;
  I: Integer;
begin
  ClearJobs;
  I := FindFirst(TheFolder[ftJobs] + '*.' + cJobExtensions[ftJobs], faArchive + faReadOnly + faHidden + faSysFile, F);
  try
    while (I = 0) do
    begin
      AddJobFromFile(F.Name);
      I := FindNext(F);
    end;
  finally
    FindClose(F);
  end;
  Result := Count;
  DoReloadJobs;
end;

procedure TNVManager.RunVampire;
begin
  if VarIsEmpty(fVampire) then fVampire := CreateOleObject(strVampireNewJob);
  try
    fVampire.Open('');
  except
    fVampire := CreateOleObject(strVampireNewJob);      // Vampire has been closed
    fVampire.Open('');
  end;
end;

procedure TNVManager.SetAutoRefresh(const Value: Boolean);
begin
  fRefreshTimer.Enabled := Value;
end;

procedure TNVManager.SetRefreshInterval(const Value: Integer);
begin
  if (fRefreshInterval <> Value) then begin
    if (Value < iMinRefresh) then fRefreshInterval := iMinRefresh
                             else fRefreshInterval := Value;
    fRefreshTimer.Interval := fRefreshInterval;
  end;
end;

function TNVManager.GetVampireRunning: Boolean;
var
  lFM: tHandle;
begin
  lFM    := OpenFileMapping(FILE_MAP_READ, False, strNV_SHARED_MEM);
  Result := (lFM <> 0);
  if Result then CloseHandle(lFM);
end;

procedure TNVManager.DoReloadJobs;
begin
  if FCanReload and Assigned(FOnReloadJobs) then FOnReloadJobs(Self);
end;

procedure TNVManager.DoDeleteJob(const AIndex: String);
begin
  if Assigned(FOnDeleteJob) then FOnDeleteJob(Self, AIndex);
end;

procedure TNVManager.DoNewJob(const AIndex: String);
begin
  if Assigned(fOnNewJob) then fOnNewJob(Self, AIndex);
end;

{ TNVJob }

constructor TNVJob.Create(aMan: TNVManager; const AIndex: string);
var
  i: tIniType;
begin
  inherited Create;
  fSaved := False;
  fMan   := aMan;
  fIndex := UpperCase(AIndex);
  for i := Low(fIniFiles) to High(fIniFiles) do
    if (i <> ftPartial) then fIniFiles[i] := tIniFile.Create(fMan.TheFolder[i] + fIndex + '._' + cJobExtensions[i]);
end;

constructor TNVJob.CreateFromFile(aMan: TNVManager; const AIndex: string);
var
  i: tIniType;
begin
  inherited Create;
  fSaved := True;
  fMan   := aMan;
  fIndex := UpperCase(AIndex);
  for i := Low(fIniFiles) to High(fIniFiles) do
    if (i <> ftPartial) then fIniFiles[i] := tIniFile.Create(fMan.TheFolder[i] + fIndex + '.' + cJobExtensions[i]);
end;

destructor TNVJob.Destroy;
var
  i: tIniType;
begin
  for i := Low(fIniFiles) to High(fIniFiles) do fIniFiles[i].Free;
  if not FSaved then
    for i := Low(tIniType) to High(tIniType) do
      DeleteFile(FMan.TheFolder[i] + FIndex + Seps[FSaved] + cJobExtensions[i]);
  inherited;
end;

function TNVJob.GetIniValue(aIniType: tIniType; aDataType: tDataType; const aSectionName, aValueName: string): Variant;
var
  S: string;
  i: Integer;
begin
  with fIniFiles[aIniType] do
    case aDataType of
      dtString : Result := ReadString (aSectionName, aValueName, '');
      dtInteger: Result := ReadInteger(aSectionName, aValueName, 0);
      dtReal   : begin
        S := ReadString (aSectionName, aValueName, '0.0');
        if (Pos(DecimalSeparator, S) < 1) then  // look for '.' or ',' and replace it with [DecimalSeparator]
          for i := 1 to Length(S) do
            if (S[i] in [',', '.']) then begin
              S[i] := DecimalSeparator;
              Break;    // don't care about other '.' or ',' -- they will be invalid in either case
            end;
        Result := StrToFloat(S);
      end;
      else Result := Null;
    end;
end;

procedure TNVJob.Save;
var
  i: tIniType;
  oldIniName,
  newIniName: String;
begin
  if fSaved then Exit;

  for i := Low(fIniFiles) to High(fIniFiles) do
  begin
    if (i = ftPartial) then Continue;
    fIniFiles[i].Free;
    oldIniName := fMan.TheFolder[i] + fIndex + '._' + cJobExtensions[i];
    NewIniName := ChangeFileExt(oldIniName, '.' + cJobExtensions[i]);
    RenameFile(oldIniName, newIniName);
    fIniFiles[i] := tIniFile.Create(newIniName);
  end;
  fSaved:=True;

  with fMan do
  begin
    DoNewJob(MyIndex);
    DoReloadJobs;
  end;
end;

procedure TNVJob.SetIniValue(aIniType: tIniType; aDataType: tDataType; const aSectionName, aValueName: string; const Value: Variant);
begin
  with fIniFiles[aIniType] do
    case aDataType of
      dtString : WriteString (aSectionName, aValueName, Value);
      dtInteger: WriteInteger(aSectionName, aValueName, Value);
      dtReal   : WriteString (aSectionName, aValueName, FloatToStr(Value));
    end;
end;

//
function FileSize(const FileName: string): Cardinal;
var
  H: THandle;
begin
  Result := 0;
  H := THandle(FileOpen(FileName, fmOpenRead + fmShareDenyNone));
  if (H <> INVALID_HANDLE_VALUE) then begin
    Result := FileSeek(H, 0, FILE_END);
    FileClose(H);
  end;
end;

function TNVJob._GetIniValue(AIndex: TJobProperty): Variant;
begin
  case AIndex of
    jpLocation                 : Result := IniValue[ftJobs , cIniNamesTypes[AIndex].rType, sIni_LOCATIONS,
                                           IniValue[ftJobs,  dtString, sIni_SETTINGS, cIniNamesTypes[AIndex].rName]];
    jpTitle..jpTag             : Result := IniValue[ftJobs , cIniNamesTypes[AIndex].rType, sIni_SETTINGS, cIniNamesTypes[AIndex].rName];
    jpState..jpIsHTML          : Result := IniValue[ftState, cIniNamesTypes[AIndex].rType, sIni_STATE,    cIniNamesTypes[AIndex].rName];
    jpPartialSize              : Result := Integer(FileSize(fMan.TheFolder[ftPartial] + fIndex + '.' + cJobExtensions[ftPartial]));
    else                         Result := Null;
  end;
  fCachedProps[AIndex] := Result;
end;

procedure TNVJob._SetIniValue(AIndex: TJobProperty; const Value: Variant);
begin
  case AIndex of
    jpLocation                 : IniValue[ftJobs,  cIniNamesTypes[AIndex].rType, sIni_LOCATIONS,
                                 IniValue[ftJobs, dtString, sIni_SETTINGS, cIniNamesTypes[AIndex].rName]] := Value;
    jpTitle..jpTag             : IniValue[ftJobs , cIniNamesTypes[AIndex].rType, sIni_SETTINGS, cIniNamesTypes[AIndex].rName] := Value;
    jpState..jpIsHTML          : IniValue[ftState, cIniNamesTypes[AIndex].rType, sIni_STATE,    cIniNamesTypes[AIndex].rName] := Value;
    jpPartialSize              : {i don't want to change the size of partial file :) };
  end;
  fCachedProps[AIndex] := Null; // to trigger the OnJobChange...
end;

// registration

procedure Register;
begin
  RegisterComponents('Samples', [TNVManager]);
end;

end.
