
{===============================================================}
{								}
{	Visual API tools					}
{	a code by Lake Unanimated / lake@lakeofsoft.com		}
{								}
{								}
{	Copyright (c) 1999, 2000 Lake of Soft			}
{	All Rights Reserved					}
{								}
{===============================================================}
{$B-}

unit VisualAPITools;

interface

uses
  Windows, Classes, Graphics, Controls;

// shows "select folder" dialog box
//
// aHandle -- handle of the owner window for the dialog box. Can be 0;
// aRoot -- location of the "root" folder to browse from.
//   Only the specified by aRoot folder and its subfolders appear in the dialog box.
//   This member can be '', and in that case, the name space root (the desktop folder) is used;
// aMsg -- string that is displayed above the tree view control in the dialog box.
//   This string can be used to specify instructions to the user;
// Result -- if user chooses the Cancel button in the dialog box, the return value is '',
//   else result is selected folder;
function SelectFolder(aHandle: tHandle; const aRoot, aMsg: string): string;

// will parse given string like this:
//
//  aString = Welcome to %NAME%, dear %USER%
//  aVChar  = %
//  aValues[0] = 'NAME=Alek'
//  aValues[1] = 'USER=Local'
//
// result:
//    Welcome to Alek, dear Local
function vParseString(const aString: string; aVChar: Char; aValues: tStrings): string;

// ...
procedure ParsePasConsts(const aConsts: string; aList: tStrings);

// too complex to work :)
function IsGoodSubStr(aSubStr: string; const aStr: string; Poss: tList; out FirstMatch: string): Boolean;

// shows message dialog with [Yes] [No] buttons
function AskYN(const aMsg: string): Boolean;

// show message dialog with [Yes] [No] [Cancel] buttons
// resturns: mrYes = Yes; mrNo = No; mrCancel = Cancel
function AskYNC(const aMsg: string): Integer;

//
function RegLoadFont(aFont: tFont; const aRegAppName: string): Boolean;

//
function RegSaveFont(aFont: tFont; const aRegAppName: string): Boolean;

//
procedure RegSaveControl(aControl: tControl; const anAppName: string);

//
function RegLoadControl(aControl: tControl; const anAppName: string): Boolean;

//
function HideShowAppFromTaskbar(DoShow: Boolean): Boolean;

implementation

uses
  SysUtils, ShlObj, ActiveX, Registry, Dialogs, Forms, CoreAPITools;

// WEIRD! Is this new Microsoft(o) style?
function SelectFolder(aHandle: tHandle; const aRoot, aMsg: string): string;
var
   lM	: iMalloc;
   lB	: Boolean;
  lB2	: Boolean;
  lBI	: TBrowseInfo;
  lDN 	: array[0..MAX_PATH] of Char;
  lDN2	: array[0..MAX_PATH] of Char;
  lWD	: pWideChar;
  lPI	: pItemIDList;
  lPI2	: pItemIDList;
  lIS	: iShellFolder;
  lEaten: uLong;
  lAttrs: uLong;
  lRL   : Integer;
begin
  Result := '';
  lB  := Succeeded(SHGetMalloc(lM));
  lRL := Length(Trim(aRoot));
  if (lRL > 0) then lB2 := Succeeded(SHGetDesktopFolder(lIS))
	       else lB2 := False;
  with lBI do begin
    hwndOwner := aHandle;
    if lB2 then begin
      GetMem(lWD, (lRL + 1) shl 1);
      try
	StringToWideChar(Trim(aRoot), lWD, lRL + 1);
	lIS.ParseDisplayName(aHandle, nil, lWD, lEaten, lPI, lAttrs);
      finally
	FreeMem(lWD, (lRL + 1) shl 1);
	pidlRoot := lPI;
      end;
    end
    else pidlRoot := nil;
    pszDisplayName := lDN;
    GetMem(lpszTitle, Length(aMsg) + 1);
    try
      StrPCopy(lpszTitle, aMsg);
      ulFlags := BIF_RETURNONLYFSDIRS;
      lpfn    := nil;
      lParam  := 0;
      lPI2    := SHBrowseForFolder(lBI);
    finally
      FreeMem(lpszTitle, Length(aMsg) + 1);
    end;
    if Assigned(lPI2) then begin
      SHGetPathFromIDList(lPI2, lDN2);
      Result := lDN2;
    end;
  end;
  //if lB2 then lIS._Release;
  if lB then begin
    lM.Free(lPI);
    if Assigned(lPI2) then lM.Free(lPI);
    lM._Release;
  end;
end;

//
function vParseString(const aString: string; aVChar: Char; aValues: tStrings): string;
var
  i: Integer;
  W: string;
  B: Boolean;
begin
  i := 1;
  Result := '';
  B := False;
  while (i <= Length(aString)) do begin
    if (aString[i] = aVChar) then
      if B then begin
	if (W = '') then Result := Result + aVChar
		    else Result := Result + aValues.Values[AnsiUpperCase(W)];
	B := False;
      end
      else begin
	B := True;
	W := '';
      end
    else
      if B then W := W + aString[i]
	   else Result := Result + aString[i];
    Inc(i);
  end;
end;

procedure ParsePasConsts(const aConsts: string; aList: tStrings);
const
  ttError	= -1;
  ttNone	= 1;
  ttID		= 2;
  ttString	= 3;
  ttDelim	= 4;

var
  lS: string;
  lP: Integer;
  lL: Integer;

  function NextToken(out lType: Integer): string;
  begin
    Result := '';
    lType  := ttNone;
    {$B-}
    // skip white space
    while (lP <= lL) and (lS[lP] in [' ', #0..#31]) do Inc(lP);
    while (lP <= lL) do begin
      case UpCase(lS[lP]) of
	'/': begin
	  lType  := ttDelim;
	  Result := '';
	  while (lP <= lL) and (lS[lP] <> #13) do Inc(lP);
	  Break;
	end;
	'{': begin
	  lType  := ttDelim;
	  Result := '';
	  while (lP <= lL) and (lS[lP] <> '}') do Inc(lP);
	  Break;
	end;
	'A'..'Z', '0'..'9', '_': begin
	  lType  := ttID;
	  Result := Result + lS[lP];
	end;
	'''': begin
	  if (Result = '') then begin
	    lType := ttString;
	    Inc(lP);
	    while (lP <= lL) do begin
	      case lS[lP] of
		'''': if (lP < lL) and (lS[lP + 1] = '''') then Result := Result + ''''
							   else Break;
		#00..#31: lType := ttError;
		else Result := Result + lS[lP];
	      end;
	      Inc(lP);
	    end;
	    Inc(lP);
	    Break;
	  end
	  else Break;
	end;
	else begin
	  if (Result = '') then begin
	    lType  := ttDelim;
	    Result := lS[lP];
	    Inc(lP);
	    Break;
	  end
	  else Break;	// token is over
	end;
      end;
      Inc(lP);
    end;
  end;

var
  lTT: Integer;
  lT : string;
  lID: string;
  lE : Boolean;
begin
  lP  := 1;
  lS  := Trim(aConsts);
  lL  := Length(lS);
  lID := '';
  lE  := False;
  repeat
    lT := NextToken(lTT);
    case lTT of
      ttNone, ttError: Break;
      ttID    : begin
	lID := lT;
	lE  := False;
      end;
      ttDelim : if not lE then lE := (lT = '=')
			  else lE := False;
      ttString: begin
	if lE then aList.Add(UpperCase(lID) + '=' + lT);
	lE := False;
      end;
    end;
  until (lTT = 0);
end;

function IsGoodSubStr(aSubStr: string; const aStr: string; Poss: tList; out FirstMatch: string): Boolean;
const
  ttWord	= 1;	//  some word (including such as "*ing" or even "*?")
  ttEOL 	= 2;	//  at end of string
  ttStar	= 3;	//  just * or ****
  Letters: set of Char = [{'''',} 'A'..'Z', 'a'..'z', '0'..'9', '_', #129..#255];

  function NextToken(const aStr: string; var aToken: string; var aPos: Integer): Integer;
  var
    lLen    : Integer;
    nonStars: Boolean;
    zPos    : Integer;
  begin
    lLen := Length(aStr);
    while (aPos <= lLen) and not ((aStr[aPos] in Letters) or (aStr[aPos] in ['*', '?'])) do Inc(aPos);
    if (aPos > lLen) then Result := ttEOL
    else begin
      aToken   := '';
      nonStars := False;
      zPos     := aPos;	// freeze the aPos
      while (zPos <= lLen) and ((aStr[zPos] in Letters) or (aStr[zPos] in ['*', '?'])) do begin
	aToken := aToken + aStr[zPos];
	if (aStr[zPos] <> '*') then nonStars := True;
	Inc(zPos);
      end;
      if nonStars then Result := ttWord
		  else Result := ttStar;
    end;
  end;

  function TokenMatch(const aMask, aToken: string): Boolean;
  var
    lP : Integer;
    lL : Integer;
    lTP: Integer;
    lTL: Integer;
    wC : Char;
    mustHave: Integer;
  begin
    lP  := 1;
    lL  := Length(aMask);
    lTP := 1;
    lTL := Length(aToken);
    while (lP <= lL) and (lTP <= lTL) do begin
      wC := aMask[lP];
      if (wC in Letters) then
	if (aToken[lTP] <> wC) then lTP := -1		// letters not match
			       else
      else
	case wC of
	  '*': begin
	    MustHave := 0;
	    while (lP <= lL) do begin
	      wC := aMask[lP];
	      case wC of
		'*': ;
		'?': Inc(mustHave);
		else Break;	// we found what we must found
	      end;
	      Inc(lP);
	    end;
	    Inc(lTP, mustHave);	// skip all "?"s
	    if (lTP > lTL) then lTP := -1	// don't have enough letters
	    else
	      if (lP > lL) then lTP := lTL	// we done, no more to check!
	      else begin
		while (lTP <= lTL) and (aToken[lTP] <> wC) do Inc(lTP);
		if (lTP > lTL) then lTP := -1;	// don't found what we must to
	      end;
	  end;
	  '?': ;	// any letter will do!
	end;
      if (lTP < 0) then Break;
      Inc(lP);
      Inc(lTP);
    end;
    Result := (lTP > lTL) and (lP > lL);
  end;

var
  lLen   : Integer;
  lSubLen: Integer;
  lPos   : Integer;
  lCurPos: Integer;
  lSubPos: Integer;
  lToken : string;
  lSubToken: string;
  CanSkip: Integer;
  lPoss  : tList;
  lTS    : Integer;
    i    : Integer;
begin
  Result     := False;
  lLen       := Length(aStr);
  aSubStr    := Trim(aSubStr);
  lSubLen    := Length(aSubStr);
  lPoss      := tList.Create;
  if Assigned(Poss) then Poss.Clear;
  lTS        := 0;		// just to eliminate warnings
  FirstMatch := '';
  if (lSubLen > 0) then begin
    lPos := 1;
    while (lPos <= lLen) do begin
      lSubPos := 1;
      lCurPos := lPos;
      CanSkip := 0;
      lPoss.Clear;
      while (lSubPos <= lSubLen) and (lCurPos > 0) do begin
	case NextToken(aSubStr, lSubToken, lSubPos) of
	  ttWord: begin
	    repeat
	      if (NextToken(aStr, lToken, lCurPos) = ttEOL) then begin
		lCurPos := -1;
		Break;
	      end
	      else begin
		lTS := lCurPos;
		Inc(lCurPos, Length(lToken));
	      end;
	      if TokenMatch(lSubToken, lToken) then begin
		lPoss.Add(Pointer(lTS));
		lPoss.Add(Pointer(Length(lToken)));
		FirstMatch := FirstMatch + ' ' + lToken;
		Break;
	      end
	      else begin
		Dec(CanSkip);
		FirstMatch := '';
	      end;
	    until (CanSkip < 0);
	    if (CanSkip < 0) then begin
	      lCurPos := -1;
	      Break;
	    end;
	    CanSkip := 0;
	  end;
	  ttEOL : ;
	  ttStar: Inc(CanSkip);
	end;
	Inc(lSubPos, Length(lSubToken));
      end;
      if (lCurPos > 0) and (lPoss.Count > 0) then begin
	if Assigned(Poss) then
	  for i := 0 to lPoss.Count - 1 do Poss.Add(lPoss[i]);
	NextToken(aStr, lToken, lCurPos);
	lPos := lCurPos;
	Result := True;
	if not Assigned(Poss) then Break;
      end
      else begin
	NextToken(aStr, lToken, lPos);
	Inc(lPos, Length(lToken));
	NextToken(aStr, lToken, lPos);
      end;
    end;
  end;
  lPoss.Free;
end;

//
function AskYN(const aMsg: string): Boolean;
begin
  Result := (MessageDlg(aMsg, mtConfirmation, [mbYes, mbNo], 0) = mrYes);
end;

//
function AskYNC(const aMsg: string): Integer;
begin
  Result := MessageDlg(aMsg, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
end;

//
function RegLoadFont(aFont: tFont; const aRegAppName: string): Boolean;
var
  lRK: string;
  lST: string;
  lF : tFont;
begin
  lRK := {AddBackSlash(}aRegAppName{)};
  //Delete(lRK, Length(lRK), 1);
  lF := tFont.Create;
  with lF do try
    CharSet := GetRegInt(lRK, 'CharSet', DEFAULT_CHARSET);
    Color   := GetRegInt(lRK, 'Color',   clWindowText);
    Height  := GetRegInt(lRK, 'Height',  -11);
    Name    := GetRegStr(lRK, 'Name',   'MS Sans Serif');
    Pitch   := tFontPitch(GetRegInt(lRK, 'Pitch',   Ord(fpDefault)));
    Size    := GetRegInt(lRK, 'Size',    8);
    lST     := GetRegStr(lRK, 'Style',   '0000');
    if (Length(lST) < 4) then lST := lST + '0000';
    if (lST[1] <> '0') then Style := Style + [fsBold];
    if (lST[2] <> '0') then Style := Style + [fsItalic];
    if (lST[3] <> '0') then Style := Style + [fsUnderline];
    if (lST[4] <> '0') then Style := Style + [fsStrikeOut];
  finally
    aFont.Assign(lF);
    Free;
  end;
  Result := True;
end;

//
function RegSaveFont(aFont: tFont; const aRegAppName: string): Boolean;
var
  lRK: string;
  lST: string;
begin
  lRK := {AddBackSlash(}aRegAppName{)};
  with aFont do begin
    SetRegInt(lRK, 'CharSet', CharSet);
    SetRegInt(lRK, 'Color',   Color);
    SetRegInt(lRK, 'Height',  Height);
    SetRegStr(lRK, 'Name',    Name);
    SetRegInt(lRK, 'Pitch',   Ord(Pitch));
    SetRegInt(lRK, 'Size',    Size);
    lST := '0000';
    if (fsBold      in Style) then lST[1] := 'A';
    if (fsItalic    in Style) then lST[2] := 'l';
    if (fsUnderline in Style) then lST[3] := 'e';
    if (fsStrikeOut in Style) then lST[4] := 'k';
    SetRegStr(lRK, 'Style', lST);
  end;
  Result := True;
end;

//
procedure RegSaveControl(aControl: tControl; const anAppName: string);
var
  lS: string;
  lF: Boolean;
begin
  with aControl do
    if (Name <> '') then begin
      lS := anAppName + '\Controls\' + Name;
      //SetRegStr(lS, 'Caption', Caption);
      //SetRegInt(lS, 'Color', Color);
      SetRegInt(lS, 'Height', Height);
      SetRegInt(lS, 'Left', Left);
      SetRegInt(lS, 'Top', Top);
      lF := (aControl is tCustomForm);
      if not lF then SetRegInt(lS, 'Visible', Ord(Visible));
      SetRegInt(lS, 'Width', Width);
      if lF then SetRegInt(lS, 'WState', Ord((aControl as tCustomForm).WindowState));
    end;
end;

//
function RegLoadControl(aControl: tControl; const anAppName: string): Boolean;
var
  lS: string;
  lF: Boolean;
begin
  with aControl do
    if (Name <> '') then begin
      lS := anAppName + '\Controls\' + Name;
      //SetRegStr(lS, 'Caption', Caption);
      //SetRegInt(lS, 'Color', Color);
      Height  := GetRegInt(lS, 'Height', Height);
      Left    := GetRegInt(lS, 'Left', Left);
      Top     := GetRegInt(lS, 'Top', Top);
      lF      := (aControl is tCustomForm);
      if not lF then Visible := (GetRegInt(lS, 'Visible', Ord(Visible)) = Ord(True));
      Width   := GetRegInt(lS, 'Width', Width);
      if lF then (aControl as tCustomForm).WindowState := tWindowState(GetRegInt(lS, 'WState', Ord((aControl as tCustomForm).WindowState)));
    end;
  Result := True;
end;

function HideShowAppFromTaskbar(DoShow: Boolean): Boolean;
var
  ExStyle: LongWord;
begin
  ExStyle := GetWindowLong(Application.Handle, GWL_EXSTYLE );
  if DoShow then ExStyle := ExStyle and not WS_EX_TOOLWINDOW
	    else ExStyle := ExStyle or WS_EX_TOOLWINDOW;
  ShowWindow(Application.Handle, SW_HIDE);
  SetWindowLong(Application.Handle, GWL_EXSTYLE, ExStyle);
  ShowWindow(Application.Handle, SW_SHOW);
  Result := True;
end;

end.

