
{===============================================================}
{								}
{	Net Vampire monitor 1.1					}
{	a code by Lake / lake@lakeofsoft.com			}
{								}
{								}
{	Copyright (c) 1999, 2000 Lake of Soft			}
{	All Rights Reserved					}
{								}
{===============================================================}

{x $DEFINE DEBUG}

unit nvMonMain;

interface

uses
  Forms, Classes, Controls, Graphics, NVMan, ComCtrls, Messages;

{$IFDEF VER110}
  {$DEFINE VER_D4}
{$ENDIF}

{$IFDEF VER120}
  {$DEFINE VER_D4}
{$ENDIF}

{$IFDEF VER125}
  {$DEFINE VER_D4}
{$ENDIF}

type
  TForm1 = class(TForm)
    procedure NVM1ReloadJobs(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    fBars    : tList;
    fMainBmp : Graphics.tBitmap;
    fCloseBmp: Graphics.tBitmap;
    NVM1     : tNVManager;
    {$IFDEF DEBUG}
    LogName  : string;
    {$ENDIF}
    procedure NVM1JobChangeState(Sender: tObject; aJob: tNVJob; const aProps: TJobProperties);
    function  GetBar(Index: Integer): tProgressBar;
    procedure WMNCHitTest(var M: TWMNCHitTest); message wm_NCHitTest;
    function  InCloseBtn(X, Y: Integer): Boolean;
    procedure ReloadJobs;
    procedure UpdateJobs;
    {$IFDEF DEBUG}
    procedure AddLog(const aData: string);
    {$ENDIF}
  protected
    procedure CreateParams(var Params: tCreateParams); override;
  public
    { Public declarations }
    constructor Create(aOwner: tComponent); override;
    property Bar[Index: Integer]: tProgressBar read GetBar;
  end;

var
  Form1: TForm1;

implementation

uses
  Windows, VisualAPITools, CoreAPITools;

{$R *.DFM}
{$R BmpRes.res}

const
  iWidth: Integer	= 60;
  ipHeight		= 6;
  regAppName 		= 'NVMonitor';

procedure TForm1.NVM1ReloadJobs(Sender: TObject);
begin
  ReloadJobs;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {$IFDEF DEBUG}
  AddLog('SHUTDOWN');
  {$ENDIF}
  RegSaveControl(Self, regAppName);
  fMainBmp.Free;
  fCloseBmp.Free;
  fBars.Free;
end;

function TForm1.GetBar(Index: Integer): tProgressBar;
begin
  Result := tProgressBar(fBars[Index]);
end;

constructor TForm1.Create(aOwner: tComponent);
begin
  fBars := tList.Create;
  fMainBmp  := Graphics.tBitmap.Create;
  fCloseBmp := Graphics.tBitmap.Create;
  fMainBmp.LoadFromResourceName(hInstance,  'MAINFRAME');
  fCloseBmp.LoadFromResourceName(hInstance, 'CLOSEBUTTON');
  inherited;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.WMNCHitTest(var M: TWMNCHitTest);
begin
  inherited;			                     	{ call the inherited message handler }
  if (M.Result = htClient) and not InCloseBtn(M.XPos - Left, M.YPos - Top) then M.Result := htCaption; 	{ is the click in the client area?   }
							{ if so, make Windows think it's     }
							{ on the caption bar.                }
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  with Canvas do begin
    StretchDraw(ClientRect, fMainBmp);
    Draw(Width - 12, 2, fCloseBmp);
  end;
end;

procedure TForm1.CreateParams(var Params: tCreateParams);
begin
  inherited;
  //Params.ExStyle := WS_EX_TOOLWINDOW;
  HideShowAppFromTaskbar(False);
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if InCloseBtn(X, Y) then Close;
end;

function TForm1.InCloseBtn(X, Y: Integer): Boolean;
begin
  Result := (X > Width - 12) and (Y > 2) and (Y < 12) and (X < Width - 2);
end;

procedure TForm1.NVM1JobChangeState(Sender: tObject; aJob: tNVJob; const aProps: TJobProperties);
begin
  if (jpState in aProps) then ReloadJobs
                         else UpdateJobs;
end;

procedure TForm1.ReloadJobs;
var
  i: Integer;
  P: tProgressBar;
begin
  for i := 0 to fBars.Count - 1 do Bar[i].Free;
  fBars.Clear;
  Height := 12;
{$IFDEF DEBUG}
  AddLog('RELOAD, COUNT = ' + IntToStr(NVM1.Count));
{$ENDIF}
  if NVM1.VampireIsRunning then
    for i := 0 to NVM1.Count - 1 do begin
      {$IFDEF DEBUG}
      AddLog('TASK #' + IntToStr(i) + ', STATUS = ' + IntToStr(Ord(tJobPhase(NVM1.JobByIndex[i][jpState]))));
      {$ENDIF}
      if (tJobPhase(NVM1.JobByIndex[i][jpState]) in [jphWait, jphConnect, jphTransfer]) then begin
	P := tProgressBar.Create(Self);
	fBars.Add(P);
	with P do begin
	  Left   := 2;
	  Height := ipHeight;
	  Width  := iWidth - 16;
	  {$IFDEF VER_D4}
	  {$ELSE}
	  Smooth := True;
	  {$ENDIF}
	  Parent := Self;
	  Self.Height := CoreAPITools.Max(8 + (fBars.Count) * ipHeight, 24);
	  Top := 6 + (fBars.Count - 1) * (Height );
	  Tag := i;
	  ShowHint := True;
	  Hint := NVM1.JobByIndex[i][jpTitle];
	end;
	{$IFDEF DEBUG}
	AddLog('TASK #' + IntToStr(i) + ' ADDED');
	{$ENDIF}
      end
      {$IFDEF DEBUG}
      else
	AddLog('TASK #' + IntToStr(i) + ' SKIPPED')
      {$ENDIF}
    end;
  {$IFDEF DEBUG}
  AddLog('ACTIVE PBARS: ' + IntToStr(fBars.Count));
  {$ENDIF}
  Self.Height := CoreAPITools.Max(8 + (fBars.Count) * ipHeight, 24);
  UpdateJobs;
  Repaint;
  Show;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  H: Integer;
begin
  {$IFDEF DEBUG}
  LogName := ExtractFilePath(ParamStr(0)) + '\nvmonlog.txt';
  AddLog('STARTUP');
  {$ENDIF}
  try
    NVM1 := tNVManager.Create(Self);
  finally
    if Assigned(NVM1) then with NVM1 do begin
      RefreshInterval      := 1000;
      AutoRefresh          := True;
      OnReloadJobs         := NVM1ReloadJobs;
      OnJobChange          := NVM1JobChangeState;
      Loaded;
    end;
  end;
  {$IFDEF DEBUG}
  AddLog('ROOT: ' + NVM1.RootFolder);
  {$ENDIF}
  H := Height;
  RegLoadControl(Self, regAppName);
  Height := H;
  Width  := iWidth;
  ReloadJobs;
end;

procedure TForm1.UpdateJobs;
var
  J: tNVJob;
  i: Integer;
  M: Integer;
  P: Integer;
  H: string;

  procedure UpdatePosition;
  begin
    with Bar[i] do
      if (M >= 100) then Position := P div (M div 100)
		    else Position := (P * 100) div M;
    {$IFDEF DEBUG}
    AddLog('PBAR #' + IntToStr(i) + ': ' + IntToStr(P) + '/' + IntToStr(M));
    {$ENDIF}
  end;

begin
  for i := 0 to fBars.Count - 1 do begin
    with Bar[i] do begin
      J := NVM1.JobByIndex[Tag];
      M := Integer(J[jpFileSize]);
      P := Integer(J[jpPartialSize]);
      H := J[jpTitle];
      if (M <= 0) then begin
	case TJobPhase(J[jpState]) of
	  jphWait	: H := H + ' - waiting';
	  jphQueue	: if (P < 1) then H := H + ' - starting'
				     else H := H + ' - ?% [' + Int2Str(P) + ' of ?]';
	  jphConnect    : H := H + ' - connecting';
	  jphTransfer   : H := H + ' - ?% [' + Int2Str(P) + ' of ?]';
	  else            H := H + ' - unknown state';
	end;
	Hint := H;
	if (P > 0) then begin
	  M := P * (Random(10) + 2);	// phunny stuff :)
	  UpdatePosition;
	end
	else Position := 0;
      end
      else begin
	UpdatePosition;
	Hint := H + ' - ' + Int2Str(Position) + '% [' + Int2Str(P) + ' of ' + Int2Str(M) + ']';
      end;
    end;
  end;
end;

{$IFDEF DEBUG}
procedure TForm1.AddLog(const aData: string);
begin
  AddTextToFile(LogName, DateTimeToStr(Now) + ' - ' + aData, True);
end;
{$ENDIF}

end.

